"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
const kinds = require("./isKind");
__export(require("./isKind"));
/**
 * If a class declaration a react class?
 * @param classDeclaration
 * @param typeChecker
 */
function isReactComponent(classDeclaration, typeChecker) {
    // Only classes that extend React.Component
    if (!classDeclaration.heritageClauses) {
        return false;
    }
    if (classDeclaration.heritageClauses.length !== 1) {
        return false;
    }
    const firstHeritageClauses = classDeclaration.heritageClauses[0];
    if (firstHeritageClauses.token !== ts.SyntaxKind.ExtendsKeyword) {
        return false;
    }
    const expressionWithTypeArguments = firstHeritageClauses.types[0];
    if (!expressionWithTypeArguments) {
        return false;
    }
    // Try type checker and fallback to node text
    const type = typeChecker.getTypeAtLocation(expressionWithTypeArguments);
    let typeSymbol = type && type.symbol && type.symbol.name;
    if (!typeSymbol) {
        typeSymbol = expressionWithTypeArguments.expression.getText();
    }
    if (!/React\.Component|Component/.test(typeSymbol)) {
        return false;
    }
    return true;
}
exports.isReactComponent = isReactComponent;
/**
 * A temporary hack until ts allows creating type alias declaration nodes
 * Creates a TypeAliasDeclaration
 * @param name Name of type node
 * @param type Type node
 * @param sourceFile SourceFile for context
 */
function createTypeDeclarationStatement(name, type, sourceFile) {
    const printer = ts.createPrinter();
    const typeString = printer.printNode(ts.EmitHint.Unspecified, type, sourceFile);
    const newSource = ts.createSourceFile('temp.ts', `type ${name} = ${typeString}`, sourceFile.languageVersion);
    return newSource.statements[0];
}
exports.createTypeDeclarationStatement = createTypeDeclarationStatement;
/**
 * Determine if a ts.HeritageClause is React HeritageClause
 *
 * @example `extends React.Component<{}, void>` is a React HeritageClause
 *
 * @todo: this is lazy. Use the typeChecker instead
 * @param clause
 */
function isReactHeritageClause(clause) {
    return clause.token === ts.SyntaxKind.ExtendsKeyword &&
        clause.types.length === 1 &&
        kinds.isExpressionWithTypeArguments(clause.types[0]) &&
        /Component/.test(clause.types[0].expression.getText());
}
exports.isReactHeritageClause = isReactHeritageClause;
/**
 * Return true if a statement is a React propType assignment statement
 * @example
 * SomeComponent.propTypes = { foo: React.PropTypes.string };
 * @param statement
 */
function isReactPropTypeAssignmentStatement(statement) {
    return kinds.isExpressionStatement(statement)
        && kinds.isBinaryExpression(statement.expression)
        && statement.expression.operatorToken.kind === ts.SyntaxKind.FirstAssignment
        && kinds.isPropertyAccessExpression(statement.expression.left)
        && /\.propTypes$|\.propTypes\..+$/.test(statement.expression.left.getText());
}
exports.isReactPropTypeAssignmentStatement = isReactPropTypeAssignmentStatement;
/**
 * Does class member have a "static" member?
 * @param classMember
 */
function hasStaticModifier(classMember) {
    if (!classMember.modifiers) {
        return false;
    }
    const staticModifier = find(classMember.modifiers, (modifier) => {
        return modifier.kind == ts.SyntaxKind.StaticKeyword;
    });
    return staticModifier !== undefined;
}
exports.hasStaticModifier = hasStaticModifier;
/**
 * Is class member a React "propTypes" member?
 * @param classMember
 * @param sourceFile
 */
function isPropTypesMember(classMember, sourceFile) {
    try {
        return classMember.name !== undefined && classMember.name.getFullText(sourceFile) !== 'propTypes';
    }
    catch (e) {
        return false;
    }
}
exports.isPropTypesMember = isPropTypesMember;
// TODO: replace following functions with Lodash?
// ---------------------------------------------------------------------------------------------------------
/**
 * Find an item in a collection with a matcher
 * @param collection
 * @param matcher
 */
function find(collection, matcher) {
    for (const item of collection) {
        if (matcher(item)) {
            return item;
        }
    }
    return undefined;
}
exports.find = find;
/**
 * Look in a collection and see if collection has a specific item
 * @param collection
 * @param matcher
 */
function has(collection, matcher) {
    if (!collection || !collection.length) {
        return false;
    }
    for (const item of collection) {
        if (matcher(item)) {
            return true;
        }
    }
    return false;
}
exports.has = has;
/**
 * Insert an item in middle of an array after a specific item
 * @param collection
 * @param afterItem
 * @param newItem
 */
function insertAfter(collection, afterItem, newItem) {
    const index = collection.indexOf(afterItem) + 1;
    return collection.slice(0, index).concat(newItem).concat(collection.slice(index));
}
exports.insertAfter = insertAfter;
/**
 * Insert an item in middle of an array before a specific item
 * @param collection
 * @param beforeItem
 * @param newItem
 */
function insertBefore(collection, beforeItem, newItem) {
    const index = collection.indexOf(beforeItem);
    return collection.slice(0, index).concat(newItem).concat(collection.slice(index));
}
exports.insertBefore = insertBefore;
/**
 * Replace an item in a collection with another item
 * @param collection
 * @param item
 * @param newItem
 */
function replaceItem(collection, item, newItem) {
    const index = collection.indexOf(item);
    return collection.slice(0, index).concat(newItem).concat(collection.slice(index + 1));
}
exports.replaceItem = replaceItem;
/**
 * Remove an item from a collection
 * @param collection
 * @param item
 * @param newItem
 */
function removeItem(collection, item) {
    const index = collection.indexOf(item);
    return collection.slice(0, index).concat(collection.slice(index + 1));
}
exports.removeItem = removeItem;
//# sourceMappingURL=index.js.map