"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
const helpers = require("../helpers");
/**
 * Hoist generics to top of a class declarations in a React component
 *
 * @example
 * Before:
 * class SomeComponent extends React.Component<{foo: number;}, {bar: string;}> {}
 *
 * After
 * type SomeComponentProps = {foo: number;};
 * type SomeComponentState = {bar: string;};
 * class SomeComponent extends React.Component<SomeComponentProps, SomeComponentState> {}
 */
function reactHoistGenericsTransformFactoryFactory(typeChecker) {
    return function reactHoistGenericsTransformFactory(context) {
        return function reactHoistGenericsTransform(node) {
            return visitSourceFile(node);
        };
    };
    function visitSourceFile(sourceFile) {
        for (const statement of sourceFile.statements) {
            if (helpers.isClassDeclaration(statement) && helpers.isReactComponent(statement, typeChecker)) {
                return hoist(statement, sourceFile);
            }
        }
        return sourceFile;
    }
}
exports.reactHoistGenericsTransformFactoryFactory = reactHoistGenericsTransformFactoryFactory;
/**
 * Hoist props and state generic types
 * @param reactClass
 * @param sourceFile
 */
function hoist(reactClass, sourceFile) {
    if (!reactClass.heritageClauses) {
        return reactClass;
    }
    const className = reactClass && reactClass.name && reactClass.name.getText(sourceFile);
    const reactHeritageClauses = helpers.find(reactClass.heritageClauses, helpers.isReactHeritageClause);
    if (reactHeritageClauses === undefined || !reactHeritageClauses.types == undefined) {
        return reactClass;
    }
    const [reactType] = reactHeritageClauses.types;
    if (reactType.typeArguments === undefined || reactType.typeArguments.length < 2) {
        return reactClass;
    }
    const [propType, stateType] = reactType.typeArguments;
    const propTypeName = `I${className}Props`;
    const stateTypeName = `I${className}State`;
    const propTypeDeclaration = helpers.createTypeDeclarationStatement(propTypeName, propType, sourceFile);
    const stateTypeDeclaration = helpers.createTypeDeclarationStatement(stateTypeName, stateType, sourceFile);
    const propTypeRef = ts.createTypeReferenceNode(propTypeName, []);
    const stateTypeRef = ts.createTypeReferenceNode(stateTypeName, []);
    const newClassStatement = insertTypeRefs(reactClass, propTypeRef, stateTypeRef);
    let statements = helpers.insertBefore(sourceFile.statements, reactClass, propTypeDeclaration);
    statements = helpers.insertAfter(statements, propTypeDeclaration, stateTypeDeclaration);
    statements = helpers.replaceItem(statements, reactClass, newClassStatement);
    return ts.updateSourceFileNode(sourceFile, statements);
}
/**
 * Replace props and state types in a React component with type references
 *
 * @example
 * input
 * ```
 * class MyComp extends React.Component<{}, {}> {}
 * ```
 *
 * output
 * ```
 * class MyComp extends React.Component<IFoo, IBar> {}
 * ```
 *
 * @param reactClassDeclaration A React class declaration
 * @param propTypeRef React Props type reference
 * @param stateTypeRef React State type reference
 */
function insertTypeRefs(reactClassDeclaration, propTypeRef, stateTypeRef) {
    if (reactClassDeclaration.heritageClauses === undefined) {
        return reactClassDeclaration;
    }
    const reactHeritageClause = helpers.find(reactClassDeclaration.heritageClauses, helpers.isReactHeritageClause);
    if (reactHeritageClause === undefined) {
        return reactClassDeclaration;
    }
    const [reactExpression] = reactHeritageClause.types;
    const newReactExpression = ts.updateExpressionWithTypeArguments(reactExpression, [propTypeRef, stateTypeRef], reactExpression.expression);
    const newHeritageClauses = ts.updateHeritageClause(reactHeritageClause, [newReactExpression]);
    return ts.updateClassDeclaration(reactClassDeclaration, reactClassDeclaration.decorators, reactClassDeclaration.modifiers, reactClassDeclaration.name, reactClassDeclaration.typeParameters, helpers.replaceItem(reactClassDeclaration.heritageClauses, reactHeritageClause, newHeritageClauses), reactClassDeclaration.members);
}
//# sourceMappingURL=react-hoist-generics-transform.js.map